@@ -103,12 +103,40 @@ static Conversion allowed_range_conversions[] = {
 	{ NULL, NULL }
 };
 
+static void
+set_secondary_index_for_single_value_property (TrackerDBInterface *iface,
+                                               const gchar        *service_name,
+                                               const gchar        *field_name,
+                                               const gchar        *second_field_name,
+                                               gboolean            enabled)
+{
+	tracker_db_interface_execute_query (iface, NULL,
+	                                    "DROP INDEX IF EXISTS \"%s_%s\"",
+	                                    service_name,
+	                                    field_name);
+
+	if (enabled) {
+		tracker_db_interface_execute_query (iface, NULL,
+		                                    "CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\", \"%s\")",
+		                                    service_name,
+		                                    field_name,
+		                                    service_name,
+		                                    field_name,
+		                                    second_field_name);
+	}
+}
+
 static void
 set_index_for_single_value_property (TrackerDBInterface *iface,
                                      const gchar        *service_name,
                                      const gchar        *field_name,
                                      gboolean            enabled)
 {
+	tracker_db_interface_execute_query (iface, NULL,
+	                                    "DROP INDEX IF EXISTS \"%s_%s\"",
+	                                    service_name,
+	                                    field_name);
+
 	if (enabled) {
 		tracker_db_interface_execute_query (iface, NULL,
 		                                    "CREATE INDEX \"%s_%s\" ON \"%s\" (\"%s\")",
@@ -116,11 +144,6 @@ set_index_for_single_value_property (TrackerDBInterface *iface,
 		                                    field_name,
 		                                    service_name,
 		                                    field_name);
-	} else {
-		tracker_db_interface_execute_query (iface, NULL,
-		                                    "DROP INDEX IF EXISTS \"%s_%s\"",
-		                                    service_name,
-		                                    field_name);
 	}
 }
 
@@ -220,7 +243,7 @@ update_property_value (const gchar     *kind,
 				needed = FALSE;
 			} else {
 
-				if (!is_allowed_conversion (str, object, allowed)) {
+				if (allowed && !is_allowed_conversion (str, object, allowed)) {
 					g_error ("Ontology change conversion not allowed '%s' -> '%s' in '%s' of '%s'",
 					         str, object, predicate, subject);
 				}
@@ -232,7 +255,11 @@ update_property_value (const gchar     *kind,
 
 			g_free (str);
 		} else {
-			needed = (g_strcmp0 (object, "true") == 0);
+			if (object && (g_strcmp0 (object, "false") == 0)) {
+				needed = FALSE;
+			} else {
+				needed = (object != NULL);
+			}
 		}
 		g_free (query);
 		if (result_set) {
@@ -312,8 +339,17 @@ fix_indexed (TrackerProperty *property)
 		set_index_for_multi_value_property (iface, service_name, field_name,
 		                                    tracker_property_get_indexed (property));
 	} else {
-		set_index_for_single_value_property (iface, service_name, field_name,
-		                                     tracker_property_get_indexed (property));
+		TrackerProperty *secondary_index;
+
+		secondary_index = tracker_property_get_secondary_index (property);
+		if (secondary_index == NULL) {
+			set_index_for_single_value_property (iface, service_name, field_name,
+			                                     tracker_property_get_indexed (property));
+		} else {
+			set_secondary_index_for_single_value_property (iface, service_name, field_name,
+			                                               tracker_property_get_name (secondary_index),
+			                                               tracker_property_get_indexed (property));
+		}
 	}
 }
 
@@ -379,6 +415,7 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 				} else {
 					/* Reset for a correct post-check */
 					tracker_property_set_indexed (property, FALSE);
+					tracker_property_set_secondary_index (property, NULL);
 					tracker_property_set_writeback (property, FALSE);
 				}
 				return;
@@ -575,6 +612,22 @@ tracker_data_ontology_load_statement (const gchar *ontology_path,
 		}
 
 		tracker_property_set_indexed (property, (strcmp (object, "true") == 0));
+	} else if (g_strcmp0 (predicate, TRACKER_PREFIX "secondaryIndex") == 0) {
+		TrackerProperty *property, *secondary_index;
+
+		property = tracker_ontologies_get_property_by_uri (subject);
+		if (property == NULL) {
+			g_critical ("%s: Unknown property %s", ontology_path, subject);
+			return;
+		}
+
+		secondary_index = tracker_ontologies_get_property_by_uri (object);
+		if (secondary_index == NULL) {
+			g_critical ("%s: Unknown property %s", ontology_path, object);
+			return;
+		}
+
+		tracker_property_set_secondary_index (property, secondary_index);
 	} else if (g_strcmp0 (predicate, TRACKER_PREFIX "transient") == 0) {
 		TrackerProperty *property;
 
@@ -707,6 +760,8 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 		for (i = 0; i < seen_properties->len; i++) {
 			TrackerProperty *property = g_ptr_array_index (seen_properties, i);
 			const gchar *subject;
+			TrackerProperty *secondary_index;
+			gboolean indexed_set = FALSE;
 
 			subject = tracker_property_get_uri (property);
 
@@ -731,6 +786,7 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 				                           "true", allowed_boolean_conversions,
 				                           NULL, property)) {
 					fix_indexed (property);
+					indexed_set = TRUE;
 				}
 			} else {
 				if (update_property_value ("tracker:indexed",
@@ -739,6 +795,29 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 				                           "false", allowed_boolean_conversions,
 				                           NULL, property)) {
 					fix_indexed (property);
+					indexed_set = TRUE;
+				}
+			}
+
+			secondary_index = tracker_property_get_secondary_index (property);
+
+			if (secondary_index) {
+				if (update_property_value ("tracker:secondaryIndex",
+				                           subject,
+				                           TRACKER_PREFIX "secondaryIndex",
+				                           tracker_property_get_uri (secondary_index), NULL,
+				                           NULL, property)) {
+					if (!indexed_set)
+						fix_indexed (property);
+				}
+			} else {
+				if (update_property_value ("tracker:secondaryIndex",
+				                           subject,
+				                           TRACKER_PREFIX "secondaryIndex",
+				                           NULL, NULL,
+				                           NULL, property)) {
+					if (!indexed_set)
+						fix_indexed (property);
 				}
 			}
 
@@ -1093,11 +1172,19 @@ class_add_super_classes_from_db (TrackerDBInterface *iface,
 {
 	TrackerDBStatement *stmt;
 	TrackerDBCursor *cursor;
+	GError *error = NULL;
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdfs:subClassOf\") "
 	                                              "FROM \"rdfs:Class_rdfs:subClassOf\" "
 	                                              "WHERE ID = (SELECT ID FROM Resource WHERE Uri = ?)");
+
+	if (!stmt) {
+		g_critical ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
+
 	tracker_db_statement_bind_text (stmt, 0, tracker_class_get_uri (class));
 	cursor = tracker_db_statement_start_cursor (stmt, NULL);
 	g_object_unref (stmt);
@@ -1122,11 +1209,19 @@ property_add_super_properties_from_db (TrackerDBInterface *iface,
 {
 	TrackerDBStatement *stmt;
 	TrackerDBCursor *cursor;
+	GError *error = NULL;
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdfs:subPropertyOf\") "
 	                                              "FROM \"rdf:Property_rdfs:subPropertyOf\" "
 	                                              "WHERE ID = (SELECT ID FROM Resource WHERE Uri = ?)");
+
+	if (!stmt) {
+		g_critical ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
+
 	tracker_db_statement_bind_text (stmt, 0, tracker_property_get_uri (property));
 	cursor = tracker_db_statement_start_cursor (stmt, NULL);
 	g_object_unref (stmt);
@@ -1149,18 +1244,22 @@ static void
 db_get_static_data (TrackerDBInterface *iface)
 {
 	TrackerDBStatement *stmt;
-	TrackerDBCursor *cursor;
+	TrackerDBCursor *cursor = NULL;
 	TrackerDBResultSet *result_set;
+	GError *error = NULL;
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"tracker:Ontology\".ID), "
 	                                              "\"nao:lastModified\" "
 	                                              "FROM \"tracker:Ontology\"");
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, &error)) {
 			TrackerOntology *ontology;
 			const gchar     *uri;
 			time_t           last_mod;
@@ -1179,17 +1278,26 @@ db_get_static_data (TrackerDBInterface *iface)
 		}
 
 		g_object_unref (cursor);
+		cursor = NULL;
 	}
 
-	stmt = tracker_db_interface_create_statement (iface,
+	if (error) {
+		g_warning ("%s", error->message);
+		g_clear_error (&error);
+	}
+
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"tracker:Namespace\".ID), "
 	                                              "\"tracker:prefix\" "
 	                                              "FROM \"tracker:Namespace\"");
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, &error)) {
 			TrackerNamespace *namespace;
 			const gchar      *uri, *prefix;
 
@@ -1208,18 +1316,27 @@ db_get_static_data (TrackerDBInterface *iface)
 		}
 
 		g_object_unref (cursor);
+		cursor = NULL;
 	}
 
-	stmt = tracker_db_interface_create_statement (iface,
+	if (error) {
+		g_warning ("%s", error->message);
+		g_clear_error (&error);
+	}
+
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT \"rdfs:Class\".ID, "
 	                                              "(SELECT Uri FROM Resource WHERE ID = \"rdfs:Class\".ID), "
 	                                              "\"tracker:notify\" "
 	                                              "FROM \"rdfs:Class\" ORDER BY ID");
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, &error)) {
 			TrackerClass *class;
 			const gchar  *uri;
 			gint          id;
@@ -1256,26 +1373,39 @@ db_get_static_data (TrackerDBInterface *iface)
 			/* xsd classes do not derive from rdfs:Resource and do not use separate tables */
 			if (!g_str_has_prefix (tracker_class_get_name (class), "xsd:")) {
 				/* update statistics */
-				stmt = tracker_db_interface_create_statement (iface, "SELECT COUNT(1) FROM \"%s\"", tracker_class_get_name (class));
-				result_set = tracker_db_statement_execute (stmt, NULL);
-				tracker_db_result_set_get (result_set, 0, &count, -1);
-				tracker_class_set_count (class, count);
-				g_object_unref (result_set);
-				g_object_unref (stmt);
+				stmt = tracker_db_interface_create_statement (iface, &error, "SELECT COUNT(1) FROM \"%s\"", tracker_class_get_name (class));
+
+				if (error) {
+					g_warning ("%s", error->message);
+					g_clear_error (&error);
+				} else {
+					result_set = tracker_db_statement_execute (stmt, NULL);
+					tracker_db_result_set_get (result_set, 0, &count, -1);
+					tracker_class_set_count (class, count);
+					g_object_unref (result_set);
+					g_object_unref (stmt);
+				}
 			}
 
 			g_object_unref (class);
 		}
 
 		g_object_unref (cursor);
+		cursor = NULL;
 	}
 
-	stmt = tracker_db_interface_create_statement (iface,
+	if (error) {
+		g_warning ("%s", error->message);
+		g_clear_error (&error);
+	}
+
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT \"rdf:Property\".ID, (SELECT Uri FROM Resource WHERE ID = \"rdf:Property\".ID), "
 	                                              "(SELECT Uri FROM Resource WHERE ID = \"rdfs:domain\"), "
 	                                              "(SELECT Uri FROM Resource WHERE ID = \"rdfs:range\"), "
 	                                              "\"nrl:maxCardinality\", "
 	                                              "\"tracker:indexed\", "
+	                                              "(SELECT Uri FROM Resource WHERE ID = \"tracker:secondaryIndex\"), "
 	                                              "\"tracker:fulltextIndexed\", "
 	                                              "\"tracker:fulltextNoLimit\", "
 	                                              "\"tracker:transient\", "
@@ -1284,14 +1414,17 @@ db_get_static_data (TrackerDBInterface *iface)
 	                                              "(SELECT 1 FROM \"rdfs:Resource_rdf:type\" WHERE ID = \"rdf:Property\".ID AND "
 	                                              "\"rdf:type\" = (SELECT ID FROM Resource WHERE Uri = '" NRL_INVERSE_FUNCTIONAL_PROPERTY "')) "
 	                                              "FROM \"rdf:Property\" ORDER BY ID");
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
+		while (tracker_db_cursor_iter_next (cursor, &error)) {
 			GValue value = { 0 };
 			TrackerProperty *property;
-			const gchar     *uri, *domain_uri, *range_uri;
+			const gchar     *uri, *domain_uri, *range_uri, *secondary_index_uri;
 			gboolean         multi_valued, indexed, fulltext_indexed, fulltext_no_limit;
 			gboolean         transient, annotation, is_inverse_functional_property;
 			gboolean         writeback;
@@ -1325,7 +1458,9 @@ db_get_static_data (TrackerDBInterface *iface)
 				indexed = FALSE;
 			}
 
-			tracker_db_cursor_get_value (cursor, 6, &value);
+			secondary_index_uri = tracker_db_cursor_get_string (cursor, 6);
+
+			tracker_db_cursor_get_value (cursor, 7, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				fulltext_indexed = (g_value_get_int (&value) == 1);
@@ -1335,7 +1470,7 @@ db_get_static_data (TrackerDBInterface *iface)
 				fulltext_indexed = FALSE;
 			}
 
-			tracker_db_cursor_get_value (cursor, 7, &value);
+			tracker_db_cursor_get_value (cursor, 8, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				fulltext_no_limit = (g_value_get_int (&value) == 1);
@@ -1345,7 +1480,7 @@ db_get_static_data (TrackerDBInterface *iface)
 				fulltext_no_limit = FALSE;
 			}
 
-			tracker_db_cursor_get_value (cursor, 8, &value);
+			tracker_db_cursor_get_value (cursor, 9, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				transient = (g_value_get_int (&value) == 1);
@@ -1355,7 +1490,7 @@ db_get_static_data (TrackerDBInterface *iface)
 				transient = FALSE;
 			}
 
-			tracker_db_cursor_get_value (cursor, 9, &value);
+			tracker_db_cursor_get_value (cursor, 10, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				annotation = (g_value_get_int (&value) == 1);
@@ -1365,8 +1500,9 @@ db_get_static_data (TrackerDBInterface *iface)
 				annotation = FALSE;
 			}
 
+
 			/* tracker:writeback column */
-			tracker_db_cursor_get_value (cursor, 10, &value);
+			tracker_db_cursor_get_value (cursor, 11, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				writeback = (g_value_get_int (&value) == 1);
@@ -1377,7 +1513,7 @@ db_get_static_data (TrackerDBInterface *iface)
 			}
 
 			/* NRL_INVERSE_FUNCTIONAL_PROPERTY column */
-			tracker_db_cursor_get_value (cursor, 11, &value);
+			tracker_db_cursor_get_value (cursor, 12, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
 				is_inverse_functional_property = TRUE;
@@ -1395,8 +1531,14 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_property_set_range (property, tracker_ontologies_get_class_by_uri (range_uri));
 			tracker_property_set_multiple_values (property, multi_valued);
 			tracker_property_set_indexed (property, indexed);
+
 			tracker_property_set_db_schema_changed (property, FALSE);
 			tracker_property_set_writeback (property, writeback);
+
+			if (secondary_index_uri) {
+				tracker_property_set_secondary_index (property, tracker_ontologies_get_property_by_uri (secondary_index_uri));
+			}
+
 			tracker_property_set_fulltext_indexed (property, fulltext_indexed);
 			tracker_property_set_fulltext_no_limit (property, fulltext_no_limit);
 			tracker_property_set_embedded (property, !annotation);
@@ -1411,6 +1553,12 @@ db_get_static_data (TrackerDBInterface *iface)
 		}
 
 		g_object_unref (cursor);
+		cursor = NULL;
+	}
+
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
 	}
 }
 
@@ -1423,11 +1571,17 @@ insert_uri_in_resource_table (TrackerDBInterface *iface,
 	TrackerDBStatement *stmt;
 	GError *error = NULL;
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "INSERT "
 	                                              "INTO Resource "
 	                                              "(ID, Uri) "
 	                                              "VALUES (?, ?)");
+	if (error) {
+		g_critical ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
+
 	tracker_db_statement_bind_int (stmt, 0, id);
 	tracker_db_statement_bind_text (stmt, 1, uri);
 	tracker_db_statement_execute (stmt, &error);
@@ -1833,7 +1987,7 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 
 	/* create index for single-valued fields */
 	for (field_it = class_properties; field_it != NULL; field_it = field_it->next) {
-		TrackerProperty *field;
+		TrackerProperty *field, *secondary_index;
 		const char   *field_name;
 
 		field = field_it->data;
@@ -1841,7 +1995,15 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 		if (!tracker_property_get_multiple_values (field)
 		    && tracker_property_get_indexed (field)) {
 			field_name = tracker_property_get_name (field);
-			set_index_for_single_value_property (iface, service_name, field_name, TRUE);
+
+			secondary_index = tracker_property_get_secondary_index (field);
+			if (secondary_index == NULL) {
+				set_index_for_single_value_property (iface, service_name, field_name, TRUE);
+			} else {
+				set_secondary_index_for_single_value_property (iface, service_name, field_name,
+				                                               tracker_property_get_name (secondary_index),
+				                                               TRUE);
+			}
 		}
 	}
 
@@ -1893,9 +2055,6 @@ create_decomposed_transient_metadata_tables (TrackerDBInterface *iface)
 
 			TrackerClass *domain;
 			const gchar *service_name;
-			const char *field_name;
-
-			field_name = tracker_property_get_name (property);
 
 			domain = tracker_property_get_domain (property);
 			service_name = tracker_class_get_name (domain);
@@ -2006,26 +2165,35 @@ get_ontologies (gboolean     test_schema,
 static gint
 get_new_service_id (TrackerDBInterface *iface)
 {
-	TrackerDBCursor    *cursor;
+	TrackerDBCursor    *cursor = NULL;
 	TrackerDBStatement *stmt;
-	gint max_service_id;
+	gint max_service_id = 0;
+	GError *error = NULL;
 
 	/* Don't intermix this thing with tracker_data_update_get_new_service_id,
 	 * if you use this, know what you are doing! */
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (iface,
+	stmt = tracker_db_interface_create_statement (iface, &error,
 	                                              "SELECT MAX(ID) AS A FROM Resource");
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
-	g_object_unref (stmt);
+
+	if (stmt) {
+		cursor = tracker_db_statement_start_cursor (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor, NULL);
+		tracker_db_cursor_iter_next (cursor, &error);
 		max_service_id = tracker_db_cursor_get_int (cursor, 0);
 		g_object_unref (cursor);
 	}
 
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
+
 	return ++max_service_id;
 }
 
@@ -2212,6 +2380,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		guint p;
 		GPtrArray *seen_classes;
 		GPtrArray *seen_properties;
+		GError *error = NULL;
 
 		seen_classes = g_ptr_array_new ();
 		seen_properties = g_ptr_array_new ();
@@ -2250,30 +2419,41 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		 * for all the ontology files in ontologies_dir whether the last-modified
 		 * has changed since we dealt with the file last time. */
 
-		stmt = tracker_db_interface_create_statement (iface,
+		stmt = tracker_db_interface_create_statement (iface, &error,
 		        "SELECT Resource.Uri, \"rdfs:Resource\".\"nao:lastModified\" FROM \"tracker:Ontology\""
 		        "INNER JOIN Resource ON Resource.ID = \"tracker:Ontology\".ID "
 		        "INNER JOIN \"rdfs:Resource\" ON \"tracker:Ontology\".ID = \"rdfs:Resource\".ID");
 
-		cursor = tracker_db_statement_start_cursor (stmt, NULL);
-		g_object_unref (stmt);
+		if (stmt) {
+			cursor = tracker_db_statement_start_cursor (stmt, &error);
+			g_object_unref (stmt);
+		} else {
+			cursor = NULL;
+		}
 
 		ontos_table = g_hash_table_new_full (g_str_hash,
 		                                     g_str_equal,
 		                                     g_free,
 		                                     NULL);
 
-		while (tracker_db_cursor_iter_next (cursor, NULL)) {
-			const gchar *onto_uri = tracker_db_cursor_get_string (cursor, 0);
-			/* It's stored as an int in the db anyway. This is caused by
-			 * string_to_gvalue in tracker-data-update.c */
-			gint value = tracker_db_cursor_get_int (cursor, 1);
+		if (cursor) {
+			while (tracker_db_cursor_iter_next (cursor, &error)) {
+				const gchar *onto_uri = tracker_db_cursor_get_string (cursor, 0);
+				/* It's stored as an int in the db anyway. This is caused by
+				 * string_to_gvalue in tracker-data-update.c */
+				gint value = tracker_db_cursor_get_int (cursor, 1);
 
-			g_hash_table_insert (ontos_table, g_strdup (onto_uri),
-			                     GINT_TO_POINTER (value));
+				g_hash_table_insert (ontos_table, g_strdup (onto_uri),
+				                     GINT_TO_POINTER (value));
+			}
+
+			g_object_unref (cursor);
 		}
 
-		g_object_unref (cursor);
+		if (error) {
+			g_warning ("%s", error->message);
+			g_clear_error (&error);
+		}
 
 		for (l = ontos; l; l = l->next) {
 			TrackerOntology *ontology;
@@ -2336,17 +2516,24 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 			if (update_nao) {
 				/* Update the nao:lastModified in the database */
-				stmt = tracker_db_interface_create_statement (iface,
+				stmt = tracker_db_interface_create_statement (iface, &error,
 				        "UPDATE \"rdfs:Resource\" SET \"nao:lastModified\"= ? "
 				        "WHERE \"rdfs:Resource\".ID = "
 				        "(SELECT Resource.ID FROM Resource INNER JOIN \"rdfs:Resource\" "
 				        "ON \"rdfs:Resource\".ID = Resource.ID WHERE "
 				        "Resource.Uri = ?)");
 
-				tracker_db_statement_bind_int (stmt, 0, last_mod);
-				tracker_db_statement_bind_text (stmt, 1, ontology_uri);
-				tracker_db_statement_execute (stmt, NULL);
-				g_object_unref (stmt);
+				if (stmt) {
+					tracker_db_statement_bind_int (stmt, 0, last_mod);
+					tracker_db_statement_bind_text (stmt, 1, ontology_uri);
+					tracker_db_statement_execute (stmt, &error);
+					g_object_unref (stmt);
+				}
+
+				if (error) {
+					g_critical ("%s", error->message);
+					g_clear_error (&error);
+				}
 			}
 
 			g_object_unref (ontology);
@@ -2354,10 +2541,10 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 		if (to_reload) {
 			/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
-			tracker_data_ontology_process_changes (seen_classes, seen_properties);
-
 			tracker_data_ontology_import_into_db (TRUE);
 
+			tracker_data_ontology_process_changes (seen_classes, seen_properties);
+
 			for (l = to_reload; l; l = l->next) {
 				const gchar *ontology_path = l->data;
 				/* store ontology in database */
@@ -2408,18 +2595,22 @@ tracker_data_manager_get_db_option_int64 (const gchar *option)
 {
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
-	TrackerDBResultSet *result_set;
+	TrackerDBResultSet *result_set = NULL;
 	gchar              *str;
 	gint                value = 0;
+	GError             *error = NULL;
 
 	g_return_val_if_fail (option != NULL, 0);
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (iface, "SELECT OptionValue FROM Options WHERE OptionKey = ?");
-	tracker_db_statement_bind_text (stmt, 0, option);
-	result_set = tracker_db_statement_execute (stmt, NULL);
-	g_object_unref (stmt);
+	stmt = tracker_db_interface_create_statement (iface, &error, "SELECT OptionValue FROM Options WHERE OptionKey = ?");
+
+	if (stmt) {
+		tracker_db_statement_bind_text (stmt, 0, option);
+		result_set = tracker_db_statement_execute (stmt, &error);
+		g_object_unref (stmt);
+	}
 
 	if (result_set) {
 		tracker_db_result_set_get (result_set, 0, &str, -1);
@@ -2432,6 +2623,12 @@ tracker_data_manager_get_db_option_int64 (const gchar *option)
 		g_object_unref (result_set);
 	}
 
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+		return -1;
+	}
+
 	return value;
 }
 
@@ -2442,20 +2639,30 @@ tracker_data_manager_set_db_option_int64 (const gchar *option,
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
 	gchar              *str;
+	GError             *error = NULL;
 
 	g_return_if_fail (option != NULL);
 
 	iface = tracker_db_manager_get_db_interface ();
 
-	stmt = tracker_db_interface_create_statement (iface, "REPLACE INTO Options (OptionKey, OptionValue) VALUES (?,?)");
-	tracker_db_statement_bind_text (stmt, 0, option);
+	stmt = tracker_db_interface_create_statement (iface, &error, "REPLACE INTO Options (OptionKey, OptionValue) VALUES (?,?)");
 
-	str = g_strdup_printf ("%"G_GINT64_FORMAT, value);
-	tracker_db_statement_bind_text (stmt, 1, str);
-	g_free (str);
+	if (stmt) {
+		tracker_db_statement_bind_text (stmt, 0, option);
 
-	tracker_db_statement_execute (stmt, NULL);
-	g_object_unref (stmt);
+		str = g_strdup_printf ("%"G_GINT64_FORMAT, value);
+		tracker_db_statement_bind_text (stmt, 1, str);
+		g_free (str);
+
+		tracker_db_statement_execute (stmt, &error);
+		g_object_unref (stmt);
+	}
+
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
 }
 
 gboolean
@@ -2463,3 +2670,9 @@ tracker_data_manager_interrupt_thread (GThread *thread)
 {
 	return tracker_db_manager_interrupt_thread (thread);
 }
+
+void
+tracker_data_manager_interrupt_thread_reset (GThread *thread)
+{
+	tracker_db_manager_interrupt_thread_reset (thread);
+}
