@@ -55,9 +55,8 @@ struct TrackerParser {
 
 	TrackerLanguage       *language;
 	gboolean               enable_stemmer;
-	guint                  max_words_to_index;
+	gboolean               enable_unaccent;
 	guint                  max_word_length;
-	gboolean               delimit_words;
 	gboolean               ignore_stop_words;
 	gboolean               ignore_reserved_words;
 	gboolean               ignore_numbers;
@@ -319,8 +318,8 @@ void
 tracker_parser_reset (TrackerParser *parser,
                       const gchar   *txt,
                       gint           txt_size,
-                      gboolean       delimit_words,
                       gboolean       enable_stemmer,
+                      gboolean       enable_unaccent,
                       gboolean       ignore_stop_words,
                       gboolean       ignore_reserved_words,
                       gboolean       ignore_numbers)
@@ -334,8 +333,8 @@ tracker_parser_reset (TrackerParser *parser,
 	g_return_if_fail (txt != NULL);
 
 	parser->enable_stemmer = enable_stemmer;
+	parser->enable_unaccent = enable_unaccent;
 	parser->ignore_stop_words = ignore_stop_words;
-	parser->delimit_words = delimit_words;
 
 	parser->txt_size = txt_size;
 	parser->txt = txt;
@@ -490,7 +489,7 @@ process_word_uchar (TrackerParser         *parser,
 	}
 
 	/* UNAC stripping needed? (for non-CJK and non-ASCII) */
-	if (type == TRACKER_PARSER_WORD_TYPE_OTHER_UNAC) {
+	if (parser->enable_unaccent && type == TRACKER_PARSER_WORD_TYPE_OTHER_UNAC) {
 		gsize stripped_word_length;
 
 		/* Get unaccented string in UTF-8 */
