@@ -35,8 +35,8 @@ TrackerParser *tracker_parser_new             (TrackerLanguage *language,
 void           tracker_parser_reset           (TrackerParser   *parser,
                                                const gchar     *txt,
                                                gint             txt_size,
-                                               gboolean         delimit_words,
                                                gboolean         enable_stemmer,
+                                               gboolean         enable_unaccent,
                                                gboolean         ignore_stop_words,
                                                gboolean         ignore_reserved_words,
                                                gboolean         ignore_numbers);
