@@ -27,6 +27,8 @@
 #include <libtracker-client/tracker-client.h>
 #include <libtracker-extract/tracker-extract.h>
 
+#define CM_TO_INCH          0.393700787
+
 typedef enum {
 	TAG_TYPE_UNDEFINED = 0,
 	TAG_TYPE_STRING,
@@ -38,26 +40,26 @@ typedef enum {
 
 typedef struct {
 	gchar *camera;
-	gchar *title;
-	gchar *orientation;
-	gchar *copyright;
-	gchar *white_balance;
-	gchar *fnumber;
-	gchar *flash;
-	gchar *focal_length;
-	gchar *artist;
-	gchar *exposure_time;
-	gchar *iso_speed_ratings;
-	gchar *date;
-	gchar *description;
-	gchar *metering_mode;
-	gchar *creator;
-	gchar *x_dimension;
-	gchar *y_dimension;
-	gchar *city;
-	gchar *state;
-	gchar *address;
-	gchar *country;
+	const gchar *title;
+	const gchar *orientation;
+	const gchar *copyright;
+	const gchar *white_balance;
+	const gchar *fnumber;
+	const gchar *flash;
+	const gchar *focal_length;
+	const gchar *artist;
+	const gchar *exposure_time;
+	const gchar *iso_speed_ratings;
+	const gchar *date;
+	const gchar *description;
+	const gchar *metering_mode;
+	const gchar *creator;
+	const gchar *x_dimension;
+	const gchar *y_dimension;
+	const gchar *city;
+	const gchar *state;
+	const gchar *address;
+	const gchar *country;
 } MergeData;
 
 typedef struct {
@@ -82,6 +84,21 @@ static TrackerExtractData extract_data[] = {
 	{ NULL, NULL }
 };
 
+static void
+tiff_data_free (TiffData *tags)
+{
+	g_free (tags->artist);
+	g_free (tags->copyright);
+	g_free (tags->date);
+	g_free (tags->title);
+	g_free (tags->description);
+	g_free (tags->width);
+	g_free (tags->length);
+	g_free (tags->make);
+	g_free (tags->model);
+	g_free (tags->orientation);
+}
+
 static gchar *
 get_flash (TIFF *image)
 {
@@ -231,56 +248,22 @@ tag_to_string (TIFF    *image,
 	return NULL;
 }
 
-static void
-insert_keywords (TrackerSparqlBuilder *metadata, 
-                 const gchar          *uri, 
-                 gchar                *keywords)
-{
-	char *lasts, *keyw;
-	size_t len;
-
-	keyw = keywords;
-	keywords = strchr (keywords, '"');
-	if (keywords) {
-		keywords++;
-	} else {
-		keywords = keyw;
-	}
-
-	len = strlen (keywords);
-	if (keywords[len - 1] == '"')
-		keywords[len - 1] = '\0';
-
-	for (keyw = strtok_r (keywords, ",;", &lasts);
-	     keyw;
-	     keyw = strtok_r (NULL, ",;", &lasts)) {
-		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
-
-		tracker_sparql_builder_object_blank_open (metadata);
-		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "nao:Tag");
-
-		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
-		tracker_sparql_builder_object_unvalidated (metadata, keyw);
-
-		tracker_sparql_builder_object_blank_close (metadata);
-	}
-}
-
 static void
 extract_tiff (const gchar          *uri,
               TrackerSparqlBuilder *preupdate,
-	      TrackerSparqlBuilder *metadata)
+              TrackerSparqlBuilder *metadata)
 {
 	TIFF *image;
-	TrackerXmpData xd = { 0 };
-	TrackerIptcData id = { 0 };
-	TrackerExifData ed = { 0 };
+	TrackerXmpData *xd = NULL;
+	TrackerIptcData *id = NULL;
+	TrackerExifData *ed = NULL;
 	MergeData md = { 0 };
 	TiffData td = { 0 };
 	gchar *filename;
 	gchar *date;
 	glong exif_offset;
+	GPtrArray *keywords;
+	guint i;
 
 #ifdef HAVE_LIBIPTCDATA
 	gchar *iptc_offset;
@@ -313,18 +296,28 @@ extract_tiff (const gchar          *uri,
 			TIFFSwabArrayOfLong((uint32*) iptc_offset, 
 			                    (unsigned long) iptc_size);
 		}
-		tracker_iptc_read (iptc_offset, 4 * iptc_size, uri, &id);
+		id = tracker_iptc_new (iptc_offset, 4 * iptc_size, uri);
 	}
 #endif /* HAVE_LIBIPTCDATA */
 
+	if (!id) {
+		id = g_new0 (TrackerIptcData, 1);
+	}
+
 	/* FIXME There are problems between XMP data embedded with different tools
 	   due to bugs in the original spec (type) */
 #ifdef HAVE_EXEMPI
 	if (TIFFGetField (image, TIFFTAG_XMLPACKET, &size, &xmp_offset)) {
-		tracker_xmp_read (xmp_offset, size, uri, &xd);
+		xd = tracker_xmp_new (xmp_offset, size, uri);
 	}
 #endif /* HAVE_EXEMPI */
 
+	if (!xd) {
+		xd = g_new0 (TrackerXmpData, 1);
+	}
+
+	ed = g_new0 (TrackerExifData, 1);
+
 	/* Get Tiff specifics */
 	td.width = tag_to_string (image, TIFFTAG_IMAGEWIDTH, TAG_TYPE_UINT32);
 	td.length = tag_to_string (image, TIFFTAG_IMAGELENGTH, TAG_TYPE_UINT32);
@@ -344,165 +337,143 @@ extract_tiff (const gchar          *uri,
 	/* Get Exif specifics */
 	if (TIFFGetField (image, TIFFTAG_EXIFIFD, &exif_offset)) {
 		if (TIFFReadEXIFDirectory (image, exif_offset)) {
-			ed.exposure_time = tag_to_string (image, EXIFTAG_EXPOSURETIME, TAG_TYPE_DOUBLE);
-			ed.fnumber = tag_to_string (image, EXIFTAG_FNUMBER, TAG_TYPE_DOUBLE);
-			ed.iso_speed_ratings = tag_to_string (image, EXIFTAG_ISOSPEEDRATINGS, TAG_TYPE_C16_UINT16);
+			ed->exposure_time = tag_to_string (image, EXIFTAG_EXPOSURETIME, TAG_TYPE_DOUBLE);
+			ed->fnumber = tag_to_string (image, EXIFTAG_FNUMBER, TAG_TYPE_DOUBLE);
+			ed->iso_speed_ratings = tag_to_string (image, EXIFTAG_ISOSPEEDRATINGS, TAG_TYPE_C16_UINT16);
 			date = tag_to_string (image, EXIFTAG_DATETIMEORIGINAL, TAG_TYPE_STRING);
-			ed.time_original = tracker_date_guess (date);
+			ed->time_original = tracker_date_guess (date);
 			g_free (date);
 
-			ed.metering_mode = get_metering_mode (image);
-			ed.flash = get_flash (image);
-			ed.focal_length = tag_to_string (image, EXIFTAG_DATETIMEORIGINAL, TAG_TYPE_DOUBLE);
-			ed.white_balance = get_white_balance (image);
-			/* ed.software = tag_to_string (image, EXIFTAG_SOFTWARE, TAG_TYPE_STRING); */
+			ed->metering_mode = get_metering_mode (image);
+			ed->flash = get_flash (image);
+			ed->focal_length = tag_to_string (image, EXIFTAG_DATETIMEORIGINAL, TAG_TYPE_DOUBLE);
+			ed->white_balance = get_white_balance (image);
+			/* ed->software = tag_to_string (image, EXIFTAG_SOFTWARE, TAG_TYPE_STRING); */
 		}
 	}
 
 	TIFFClose (image);
 	g_free (filename);
 
-	md.camera = tracker_merge (" ", 2, xd.make, xd.model);
+	md.camera = tracker_merge_const (" ", 2, xd->make, xd->model);
 
 	if (!md.camera) {
-		md.camera = tracker_merge (" ", 2, td.make, td.model);
+		md.camera = tracker_merge_const (" ", 2, td.make, td.model);
 
 		if (!md.camera) {
-			md.camera = tracker_merge (" ", 2, ed.make, ed.model);
-		} else {
-			g_free (ed.model);
-			g_free (ed.make);
+			md.camera = tracker_merge_const (" ", 2, ed->make, ed->model);
 		}
-	} else {
-		g_free (td.model);
-		g_free (td.make);
-		g_free (ed.model);
-		g_free (ed.make);
 	}
 
-	md.title = tracker_coalesce (5, xd.title, xd.pdf_title, td.title, ed.document_name, xd.title2);
-	md.orientation = tracker_coalesce (4, xd.orientation, td.orientation, ed.orientation, id.image_orientation);
-	md.copyright = tracker_coalesce (4, xd.rights, td.copyright, ed.copyright, id.copyright_notice);
-	md.white_balance = tracker_coalesce (2, xd.white_balance, ed.white_balance);
-	md.fnumber = tracker_coalesce (2, xd.fnumber, ed.fnumber);
-	md.flash = tracker_coalesce (2, xd.flash, ed.flash);
-	md.focal_length = tracker_coalesce (2, xd.focal_length, ed.focal_length);
-	md.artist = tracker_coalesce (4, xd.artist, td.artist, ed.artist, xd.contributor);
-	md.exposure_time = tracker_coalesce (2, xd.exposure_time, ed.exposure_time);
-	md.iso_speed_ratings = tracker_coalesce (2, xd.iso_speed_ratings, ed.iso_speed_ratings);
-	md.date = tracker_coalesce (6, xd.date, xd.time_original, td.date, ed.time, id.date_created, ed.time_original);
-	md.description = tracker_coalesce (3, xd.description, td.description, ed.description);
-	md.metering_mode = tracker_coalesce (2, xd.metering_mode, ed.metering_mode);
-	md.city = tracker_coalesce (2, xd.city, id.city);
-	md.state = tracker_coalesce (2, xd.state, id.state);
-	md.address = tracker_coalesce (2, xd.address, id.sublocation);
-	md.country = tracker_coalesce (2, xd.country, id.country_name);
-	md.creator = tracker_coalesce (3, xd.creator, id.byline, id.credit);
-	md.x_dimension = tracker_coalesce (2, td.width, ed.x_dimension);
-	md.y_dimension = tracker_coalesce (2, td.length, ed.y_dimension);
+	md.title = tracker_coalesce_strip (5, xd->title, xd->pdf_title, td.title, ed->document_name, xd->title2);
+	md.orientation = tracker_coalesce_strip (4, xd->orientation, td.orientation, ed->orientation, id->image_orientation);
+	md.copyright = tracker_coalesce_strip (4, xd->rights, td.copyright, ed->copyright, id->copyright_notice);
+	md.white_balance = tracker_coalesce_strip (2, xd->white_balance, ed->white_balance);
+	md.fnumber = tracker_coalesce_strip (2, xd->fnumber, ed->fnumber);
+	md.flash = tracker_coalesce_strip (2, xd->flash, ed->flash);
+	md.focal_length = tracker_coalesce_strip (2, xd->focal_length, ed->focal_length);
+	md.artist = tracker_coalesce_strip (4, xd->artist, td.artist, ed->artist, xd->contributor);
+	md.exposure_time = tracker_coalesce_strip (2, xd->exposure_time, ed->exposure_time);
+	md.iso_speed_ratings = tracker_coalesce_strip (2, xd->iso_speed_ratings, ed->iso_speed_ratings);
+	md.date = tracker_coalesce_strip (6, xd->date, xd->time_original, td.date, ed->time, id->date_created, ed->time_original);
+	md.description = tracker_coalesce_strip (3, xd->description, td.description, ed->description);
+	md.metering_mode = tracker_coalesce_strip (2, xd->metering_mode, ed->metering_mode);
+	md.city = tracker_coalesce_strip (2, xd->city, id->city);
+	md.state = tracker_coalesce_strip (2, xd->state, id->state);
+	md.address = tracker_coalesce_strip (2, xd->address, id->sublocation);
+	md.country = tracker_coalesce_strip (2, xd->country, id->country_name);
+	md.creator = tracker_coalesce_strip (3, xd->creator, id->byline, id->credit);
+	md.x_dimension = tracker_coalesce_strip (2, td.width, ed->x_dimension);
+	md.y_dimension = tracker_coalesce_strip (2, td.length, ed->y_dimension);
 
-	if (ed.user_comment) {
+	keywords = g_ptr_array_new ();
+
+	if (ed->user_comment) {
 		tracker_sparql_builder_predicate (metadata, "nie:comment");
-		tracker_sparql_builder_object_unvalidated (metadata, ed.user_comment);
-		g_free (ed.user_comment);
+		tracker_sparql_builder_object_unvalidated (metadata, ed->user_comment);
 	}
 
 	if (md.x_dimension) {
 		tracker_sparql_builder_predicate (metadata, "nfo:width");
 		tracker_sparql_builder_object_unvalidated (metadata, md.x_dimension);
-		g_free (md.x_dimension);
 	}
 
 	if (md.y_dimension) {
 		tracker_sparql_builder_predicate (metadata, "nfo:height");
 		tracker_sparql_builder_object_unvalidated (metadata, md.y_dimension);
-		g_free (md.y_dimension);
 	}
 
-	if (xd.keywords) {
-		insert_keywords (metadata, uri, xd.keywords);
-		g_free (xd.keywords);
+	if (xd->keywords) {
+		tracker_keywords_parse (keywords, xd->keywords);
 	}
 
-	if (xd.pdf_keywords) {
-		insert_keywords (metadata, uri, xd.pdf_keywords);
-		g_free (xd.pdf_keywords);
+	if (xd->pdf_keywords) {
+		tracker_keywords_parse (keywords, xd->pdf_keywords);
 	}
 
-	if (xd.subject) {
-		insert_keywords (metadata, uri, xd.subject);
-		g_free (xd.subject);
+	if (xd->subject) {
+		tracker_keywords_parse (keywords, xd->subject);
 	}
 
-	if (xd.publisher) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", xd.publisher);
+	if (xd->publisher) {
+		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", xd->publisher);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nco:Contact");
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
-		tracker_sparql_builder_object_unvalidated (preupdate, xd.publisher);
+		tracker_sparql_builder_object_unvalidated (preupdate, xd->publisher);
 		tracker_sparql_builder_insert_close (preupdate);
-		g_free (xd.publisher);
 
 		tracker_sparql_builder_predicate (metadata, "nco:publisher");
 		tracker_sparql_builder_object_iri (metadata, uri);
 		g_free (uri);
 	}
 
-	if (xd.type) {
+	if (xd->type) {
 		tracker_sparql_builder_predicate (metadata, "dc:type");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.type);
-		g_free (xd.type);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->type);
 	}
 
-	if (xd.format) {
+	if (xd->format) {
 		tracker_sparql_builder_predicate (metadata, "dc:format");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.format);
-		g_free (xd.format);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->format);
 	}
 
-	if (xd.identifier) {
+	if (xd->identifier) {
 		tracker_sparql_builder_predicate (metadata, "dc:identifier");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.identifier);
-		g_free (xd.identifier);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->identifier);
 	}
 
-	if (xd.source) {
+	if (xd->source) {
 		tracker_sparql_builder_predicate (metadata, "dc:source");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.source);
-		g_free (xd.source);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->source);
 	}
 
-	if (xd.language) {
+	if (xd->language) {
 		tracker_sparql_builder_predicate (metadata, "dc:language");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.language);
-		g_free (xd.language);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->language);
 	}
 
-	if (xd.relation) {
+	if (xd->relation) {
 		tracker_sparql_builder_predicate (metadata, "dc:relation");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.relation);
-		g_free (xd.relation);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->relation);
 	}
 
-	if (xd.coverage) {
+	if (xd->coverage) {
 		tracker_sparql_builder_predicate (metadata, "dc:coverage");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.coverage);
-		g_free (xd.coverage);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->coverage);
 	}
 
-	if (xd.rating) {
+	if (xd->rating) {
 		tracker_sparql_builder_predicate (metadata, "nao:numericRating");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.rating);
-		g_free (xd.rating);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->rating);
 	}
 
-	if (xd.license) {
+	if (xd->license) {
 		tracker_sparql_builder_predicate (metadata, "nie:license");
-		tracker_sparql_builder_object_unvalidated (metadata, xd.license);
-		g_free (xd.license);
+		tracker_sparql_builder_object_unvalidated (metadata, xd->license);
 	}
 
 	if (md.city || md.state || md.address || md.country) {
@@ -511,102 +482,105 @@ extract_tiff (const gchar          *uri,
 		tracker_sparql_builder_object_blank_open (metadata);
 		tracker_sparql_builder_predicate (metadata, "a");
 		tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
-	
+
 		if (md.address) {
 			tracker_sparql_builder_predicate (metadata, "mlo:address");
 			tracker_sparql_builder_object_unvalidated (metadata, md.address);
-			g_free (md.address);
 		}
-	
+
 		if (md.state) {
 			tracker_sparql_builder_predicate (metadata, "mlo:state");
 			tracker_sparql_builder_object_unvalidated (metadata, md.state);
-			g_free (md.state);
 		}
-	
+
 		if (md.city) {
 			tracker_sparql_builder_predicate (metadata, "mlo:city");
 			tracker_sparql_builder_object_unvalidated (metadata, md.city);
-			g_free (md.city);
 		}
-	
+
 		if (md.country) {
 			tracker_sparql_builder_predicate (metadata, "mlo:country");
 			tracker_sparql_builder_object_unvalidated (metadata, md.country);
-			g_free (md.country);
 		}
-		
+
 		tracker_sparql_builder_object_blank_close (metadata);
 	}
 
-	if (id.contact) {
-		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", id.contact);
+	if (id->contact) {
+		gchar *uri = tracker_uri_printf_escaped ("urn:contact:%s", id->contact);
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, uri);
 		tracker_sparql_builder_predicate (preupdate, "a");
 		tracker_sparql_builder_object (preupdate, "nco:Contact");
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
-		tracker_sparql_builder_object_unvalidated (preupdate, id.contact);
+		tracker_sparql_builder_object_unvalidated (preupdate, id->contact);
 		tracker_sparql_builder_insert_close (preupdate);
-		g_free (id.contact);
 
 		tracker_sparql_builder_predicate (metadata, "nco:representative");
 		tracker_sparql_builder_object_iri (metadata, uri);
 		g_free (uri);
 	}
 
-	if (id.keywords) {
-		insert_keywords (metadata, uri, id.keywords);
-		g_free (id.keywords);
+	if (id->keywords) {
+		tracker_keywords_parse (keywords, id->keywords);
 	}
 
+	for (i = 0; i < keywords->len; i++) {
+		gchar *p;
+
+		p = g_ptr_array_index (keywords, i);
+
+		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
+
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nao:Tag");
+		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
+		tracker_sparql_builder_object_unvalidated (metadata, p);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (p);
+	}
+	g_ptr_array_free (keywords, TRUE);
+
 	if (md.camera) {
 		tracker_sparql_builder_predicate (metadata, "nmm:camera");
 		tracker_sparql_builder_object_unvalidated (metadata, md.camera);
-		g_free (md.camera);
 	}
 
 	if (md.title) {
 		tracker_sparql_builder_predicate (metadata, "nie:title");
 		tracker_sparql_builder_object_unvalidated (metadata, md.title);
-		g_free (md.title);
 	}
 
 	if (md.orientation) {
 		tracker_sparql_builder_predicate (metadata, "nfo:orientation");
 		tracker_sparql_builder_object_unvalidated (metadata, md.orientation);
-		g_free (md.orientation);
 	}
 
 	if (md.copyright) {
 		tracker_sparql_builder_predicate (metadata, "nie:copyright");
 		tracker_sparql_builder_object_unvalidated (metadata, md.copyright);
-		g_free (md.copyright);
 	}
 
 	if (md.white_balance) {
 		tracker_sparql_builder_predicate (metadata, "nmm:whiteBalance");
 		tracker_sparql_builder_object_unvalidated (metadata, md.white_balance);
-		g_free (md.white_balance);
 	}
 
 	if (md.fnumber) {
 		tracker_sparql_builder_predicate (metadata, "nmm:fnumber");
 		tracker_sparql_builder_object_unvalidated (metadata, md.fnumber);
-		g_free (md.fnumber);
 	}
 
 	if (md.flash) {
 		tracker_sparql_builder_predicate (metadata, "nmm:flash");
 		tracker_sparql_builder_object_unvalidated (metadata, md.flash);
-		g_free (md.flash);
 	}
 
 	if (md.focal_length) {
 		tracker_sparql_builder_predicate (metadata, "nmm:focalLength");
 		tracker_sparql_builder_object_unvalidated (metadata, md.focal_length);
-		g_free (md.focal_length);
 	}
 
 	if (md.artist) {
@@ -619,7 +593,6 @@ extract_tiff (const gchar          *uri,
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.artist);
 		tracker_sparql_builder_insert_close (preupdate);
-		g_free (md.artist);
 
 		tracker_sparql_builder_predicate (metadata, "nco:contributor");
 		tracker_sparql_builder_object_iri (metadata, uri);
@@ -629,31 +602,26 @@ extract_tiff (const gchar          *uri,
 	if (md.exposure_time) {
 		tracker_sparql_builder_predicate (metadata, "nmm:exposureTime");
 		tracker_sparql_builder_object_unvalidated (metadata, md.exposure_time);
-		g_free (md.exposure_time);
 	}
 
 	if (md.iso_speed_ratings) {
 		tracker_sparql_builder_predicate (metadata, "nmm:isoSpeed");
 		tracker_sparql_builder_object_unvalidated (metadata, md.iso_speed_ratings);
-		g_free (md.iso_speed_ratings);
 	}
 
 	if (md.date) {
 		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
 		tracker_sparql_builder_object_unvalidated (metadata, md.date);
-		g_free (md.date);
 	}
 
 	if (md.description) {
 		tracker_sparql_builder_predicate (metadata, "nie:description");
 		tracker_sparql_builder_object_unvalidated (metadata, md.description);
-		g_free (md.description);
 	}
 
 	if (md.metering_mode) {
 		tracker_sparql_builder_predicate (metadata, "nmm:meteringMode");
 		tracker_sparql_builder_object_unvalidated (metadata, md.metering_mode);
-		g_free (md.metering_mode);
 	}
 
 	if (md.creator) {
@@ -666,14 +634,13 @@ extract_tiff (const gchar          *uri,
 		tracker_sparql_builder_predicate (preupdate, "nco:fullname");
 		tracker_sparql_builder_object_unvalidated (preupdate, md.creator);
 		tracker_sparql_builder_insert_close (preupdate);
-		g_free (md.creator);
 
 		/* NOTE: We only have affiliation with
 		 * nco:PersonContact and we are using
 		 * nco:Contact here.
 		 */
 
-		/* if (id.byline_title) { */
+		/* if (id->byline_title) { */
 		/* 	tracker_sparql_builder_insert_open (preupdate, NULL); */
 
 		/* 	tracker_sparql_builder_subject (preupdate, "_:affiliation_by_line"); */
@@ -681,7 +648,7 @@ extract_tiff (const gchar          *uri,
 		/* 	tracker_sparql_builder_object (preupdate, "nco:Affiliation"); */
 
 		/* 	tracker_sparql_builder_predicate (preupdate, "nco:title"); */
-		/* 	tracker_sparql_builder_object_unvalidated (preupdate, id.byline_title); */
+		/* 	tracker_sparql_builder_object_unvalidated (preupdate, id->byline_title); */
 
 		/* 	tracker_sparql_builder_insert_close (preupdate); */
 
@@ -696,7 +663,27 @@ extract_tiff (const gchar          *uri,
 		g_free (uri);
 	}
 
-	g_free (id.byline_title);
+	if (ed->x_resolution) {
+		gdouble value;
+
+		value = ed->resolution_unit == 1 ? g_strtod (ed->x_resolution, NULL) : g_strtod (ed->x_resolution, NULL) * CM_TO_INCH;
+		tracker_sparql_builder_predicate (metadata, "nfo:horizontalResolution");
+		tracker_sparql_builder_object_double (metadata, value);
+	}
+
+	if (ed->y_resolution) {
+		gdouble value;
+
+		value = ed->resolution_unit == 1 ? g_strtod (ed->y_resolution, NULL) : g_strtod (ed->y_resolution, NULL) * CM_TO_INCH;
+		tracker_sparql_builder_predicate (metadata, "nfo:verticalResolution");
+		tracker_sparql_builder_object_double (metadata, value);
+	}
+
+	g_free (md.camera);
+	tiff_data_free (&td);
+	tracker_exif_free (ed);
+	tracker_xmp_free (xd);
+	tracker_iptc_free (id);
 }
 
 TrackerExtractData *
